require_dependency "cally/application_controller"

module Cally
  class InvitationsController < ApplicationController
  
    require 'omw_random_string'
    require 'omw_mailgun'

    before_action :is_invitation_only?, only: [:new, :create]
    before_action :invitation_params, only: [:create]
    before_action :logged_in_as_admin?, only: [:index, :destroy, :send_invitation]

    def new
      # if this website is invitation only then you can enter an e-mail address to request
      # an invitation. Else it redirects to the signup path.
      if @invitation_only == 'true'
        @invitation = Invitation.new
      else
        flash[:error] = 'This is an open subscription, no invitation needed.'
        redirect_to new_user_path
      end
    end

    def create
      # Here it will add the e-mail address and a random generated key into the database.
      # If this is not invitation only then redirects to the signup path.
      if @invitation_only == 'true'
        @invitation = Invitation.new(invitation_params)
        @invitation.invitation_key = OmwRandomString.generate(32)

        if @invitation.save
          flash[:success] = "Created invitation request for '#{@invitation.email}'."
          redirect_to login_path
        else
          flash[:error] = 'Error create invitation request.'
          render 'new'
        end
      else
        flash[:error] = 'This is an open subscription, no invitation needed.'
        redirect_to new_user_path
      end
    end
    
    def index
      # Shows all the pending invitations. 20 per page
      @invitations = Invitation.paginate(page: params[:page], per_page: 20)
    end

    def destroy
      # removes a invitation from the database.
      invite = Invitation.find(params[:id])

      if invite.destroy
        flash[:success] = 'Invitation successfully deleted.'
        redirect_to all_invitations_path
      else
        flash[:error] = 'Error deleting invitation.'
        redirect_to all_invitations_path
      end
    end

    def send_invitation
      # Here you can send the created invitation the the user's e-mail address given when created
      # a new invitation.

      # set_mailgun_prefix first checks if a custom prefix is set in an environment variable.
      # If not it uses the default as given in lib/cally/methods.rb
      set_mailgun_prefix
      invitation = Invitation.find(params[:id])

      # The invitation url is the url which is send to the user's e-mail address given when creating a new
      # invitation. 
      # If the user clicks on the url in his received e-mail it take's the user
      # to the signup page where he/she can fill his/her information and create a password.
      invitation_url = request.base_url + "/#{@mailgun_prefix}invitation/#{invitation.invitation_key}"

      if invitation.update(sent: true) && send_invite(invitation.email, invitation_url) 
        flash[:success] = "Sent invitation to '#{invitation.email}'" 
        redirect_to all_invitations_path
      else
        flash[:error] = "Error sending invitation url to '#{invitation.email}'"
        redirect_to all_invitations_path
      end
    end

    private

    def invitation_params
      params.require(:invitation).permit(:email)
    end

    def is_invitation_only?
      @invitation_only = ENV['INVITATION_ONLY'] || 'false'
    end

    def send_invite(email, invitation_url)
      if !test_env?
        # If it's not in testing environment then send e-mail
        # There are 3 required environment variables here, put them and their value in the .env file
        # in the root folder of the app which is using this cally engine
        # MAILGUN_DOMAIN
        # MAILGUN_SENDER_NAME
        # MAILGUN_SENDER_EMAIL
        res = Mailgun.post("/#{ENV['MAILGUN_DOMAIN']}/messages", body: { 
          from: "#{ENV['MAILGUN_SENDER_NAME']} <#{ENV['MAILGUN_SENDER_EMAIL']}>",
          to: "#{email}", subject: "You're invited for onemanweb", text: "invitation link: #{invitation_url}" })

        if res.code == 200
          return true
        else
          @error_send_message = res.message
          return false
        end
      elsif test_env?
        return true
      else
        return false
      end
    end

  end
end
